@@ -15,18 +15,18 @@ use crate::{
 use futures::channel::oneshot;
 use futures::FutureExt;
 use futures::StreamExt;
-use log::{info, warn};
 use tor_dirclient::DirResponse;
 use tor_rtcompat::{Runtime, SleepProviderExt};
+use tracing::{info, trace, warn};
 
 /// Try to read a set of documents from `dirmgr` by ID.
-async fn load_all<R: Runtime>(
+fn load_all<R: Runtime>(
     dirmgr: &DirMgr<R>,
     missing: Vec<DocId>,
 ) -> Result<HashMap<DocId, DocumentText>> {
     let mut loaded = HashMap::new();
     for query in docid::partition_by_type(missing.into_iter()).values() {
-        dirmgr.load_documents_into(query, &mut loaded).await?;
+        dirmgr.load_documents_into(query, &mut loaded)?;
     }
     Ok(loaded)
 }
@@ -39,7 +39,7 @@ async fn fetch_single<R: Runtime>(
     let circmgr = dirmgr.circmgr()?;
     let cur_netdir = dirmgr.opt_netdir();
     let dirinfo = match cur_netdir {
-        Some(ref nd) => nd.as_ref().into(),
+        Some(ref netdir) => netdir.as_ref().into(),
         None => dirmgr.config.fallbacks().into(),
     };
     let resource =
@@ -60,7 +60,7 @@ async fn fetch_multiple<R: Runtime>(
 ) -> Result<Vec<(ClientRequest, DirResponse)>> {
     let mut requests = Vec::new();
     for (_type, query) in docid::partition_by_type(missing.into_iter()) {
-        requests.extend(dirmgr.query_into_requests(query).await?);
+        requests.extend(dirmgr.query_into_requests(query)?);
     }
 
     // TODO: instead of waiting for all the queries to finish, we
@@ -90,12 +90,19 @@ async fn load_once<R: Runtime>(
     state: &mut Box<dyn DirState>,
 ) -> Result<bool> {
     let missing = state.missing_docs();
-    if missing.is_empty() {
+    let outcome = if missing.is_empty() {
+        trace!("Found no missing documents; can't advance current state");
         Ok(false)
     } else {
-        let documents = load_all(dirmgr, missing).await?;
-        state.add_from_cache(documents)
-    }
+        trace!(
+            "Found {} missing documents; trying to load them",
+            missing.len()
+        );
+        let documents = load_all(dirmgr, missing)?;
+        state.add_from_cache(documents, dirmgr.store_if_rw())
+    };
+    dirmgr.notify().await;
+    outcome
 }
 
 /// Try to load as much state as possible for a provided `state` from the
@@ -108,19 +115,22 @@ pub(crate) async fn load<R: Runtime>(
 ) -> Result<Box<dyn DirState>> {
     let mut safety_counter = 0_usize;
     loop {
+        trace!(state=%state.describe(), "Loading from cache");
         let changed = load_once(&dirmgr, &mut state).await?;
 
         if state.can_advance() {
             state = state.advance()?;
+            dirmgr.notify().await;
             safety_counter = 0;
         } else {
             if !changed {
                 break;
             }
             safety_counter += 1;
-            if safety_counter == 100 {
-                panic!("Spent 100 iterations in the same state: this is a bug");
-            }
+            assert!(
+                safety_counter < 100,
+                "Spent 100 iterations in the same state: this is a bug"
+            );
         }
     }
 
@@ -143,12 +153,11 @@ async fn download_attempt<R: Runtime>(
     let missing = state.missing_docs();
     let fetched = fetch_multiple(Arc::clone(dirmgr), missing, parallelism).await?;
     for (client_req, dir_response) in fetched {
-        let text = dir_response.into_output();
-        match dirmgr.expand_response_text(&client_req, text).await {
+        let text = String::from_utf8(dir_response.into_output())?;
+        match dirmgr.expand_response_text(&client_req, text) {
             Ok(text) => {
-                let outcome = state
-                    .add_from_download(&text, &client_req, Some(&dirmgr.store))
-                    .await;
+                let outcome = state.add_from_download(&text, &client_req, Some(&dirmgr.store));
+                dirmgr.notify().await;
                 match outcome {
                     Ok(b) => changed |= b,
                     // TODO: in this case we might want to stop using this source.
@@ -182,7 +191,7 @@ async fn download_attempt<R: Runtime>(
 pub(crate) async fn download<R: Runtime>(
     dirmgr: Weak<DirMgr<R>>,
     mut state: Box<dyn DirState>,
-    mut on_usable: Option<oneshot::Sender<()>>,
+    on_usable: &mut Option<oneshot::Sender<()>>,
 ) -> Result<(Box<dyn DirState>, Option<Error>)> {
     let runtime = upgrade_weak_ref(&dirmgr)?.runtime.clone();
 
@@ -224,7 +233,9 @@ pub(crate) async fn download<R: Runtime>(
                                 warn!("Error while downloading: {}", e);
                                 continue 'next_attempt;
                             }
-                            Ok(changed) => changed
+                            Ok(changed) => {
+                                changed
+                            }
                         }
                     }
                     _ = runtime.sleep_until_wallclock(reset_time).fuse() => {
@@ -245,12 +256,15 @@ pub(crate) async fn download<R: Runtime>(
 
             // Report usable-ness if appropriate.
             if on_usable.is_some() && state.is_ready(Readiness::Usable) {
+                // Unwrap should be safe due to parent `.is_some()` check
+                #[allow(clippy::unwrap_used)]
                 let _ = on_usable.take().unwrap().send(());
             }
 
             if state.can_advance() {
                 // We have enough info to advance to another state.
                 state = state.advance()?;
+                upgrade_weak_ref(&dirmgr)?.notify().await;
                 continue 'next_state;
             } else {
                 // We should wait a bit, and then retry.
@@ -268,6 +282,9 @@ pub(crate) async fn download<R: Runtime>(
         }
 
         // We didn't advance the state, after all the retries.
+        warn!(n_attempts=retry_config.n_attempts(),
+              state=%state.describe(),
+              "Unable to advance downloading state");
         return Ok((state, Some(Error::CantAdvanceState)));
     }
 }
