@@ -13,12 +13,21 @@
 //! This directory manager crate is responsible for figuring out which
 //! directory information we lack, downloading what we're missing, and
 //! keeping a cache of it on disk.
+//!
+//! # Compile-time features
+//!
+//! `mmap` (default) -- Use memory mapping to reduce the memory load for
+//! reading large directory objects from disk.
+//!
+//! `static` -- Try to link with a static copy of sqlite3.
 
 #![deny(missing_docs)]
 #![warn(noop_method_call)]
 #![deny(unreachable_pub)]
 #![deny(clippy::await_holding_lock)]
 #![deny(clippy::cargo_common_metadata)]
+#![deny(clippy::cast_lossless)]
+#![deny(clippy::checked_conversions)]
 #![warn(clippy::clone_on_ref_ptr)]
 #![warn(clippy::cognitive_complexity)]
 #![deny(clippy::debug_assert_with_mut_call)]
@@ -26,14 +35,20 @@
 #![deny(clippy::exhaustive_structs)]
 #![deny(clippy::expl_impl_clone_on_copy)]
 #![deny(clippy::fallible_impl_from)]
+#![deny(clippy::implicit_clone)]
 #![deny(clippy::large_stack_arrays)]
 #![warn(clippy::manual_ok_or)]
 #![deny(clippy::missing_docs_in_private_items)]
+#![deny(clippy::missing_panics_doc)]
+#![warn(clippy::needless_borrow)]
+#![warn(clippy::needless_pass_by_value)]
 #![warn(clippy::option_option)]
 #![warn(clippy::rc_buffer)]
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::trait_duplication_in_bounds)]
+#![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
+#![deny(clippy::unwrap_used)]
 
 pub mod authority;
 mod bootstrap;
@@ -41,6 +56,7 @@ mod config;
 mod docid;
 mod docmeta;
 mod err;
+mod event;
 mod retry;
 mod shared_ref;
 mod state;
@@ -54,24 +70,28 @@ use tor_circmgr::CircMgr;
 use tor_netdir::NetDir;
 use tor_netdoc::doc::netstatus::ConsensusFlavor;
 
-use anyhow::{Context, Result};
-use async_trait::async_trait;
-use futures::{channel::oneshot, lock::Mutex, task::SpawnExt};
-use log::{info, warn};
+use futures::{channel::oneshot, task::SpawnExt};
 use tor_rtcompat::{Runtime, SleepProviderExt};
+use tracing::{info, trace, warn};
 
-use std::sync::Arc;
+use std::sync::atomic::{AtomicBool, Ordering};
+use std::sync::{Arc, Mutex};
 use std::{collections::HashMap, sync::Weak};
 use std::{fmt::Debug, time::SystemTime};
 
-pub use authority::Authority;
+pub use authority::{Authority, AuthorityBuilder};
 pub use config::{
     DirMgrConfig, DirMgrConfigBuilder, DownloadScheduleConfig, DownloadScheduleConfigBuilder,
     NetworkConfig, NetworkConfigBuilder,
 };
 pub use docid::DocId;
 pub use err::Error;
+pub use event::DirEvent;
 pub use storage::DocumentText;
+pub use tor_netdir::fallback::{FallbackDir, FallbackDirBuilder};
+
+/// A Result as returned by this crate.
+pub type Result<T> = std::result::Result<T, Error>;
 
 /// A directory manager to download, fetch, and cache a Tor directory.
 ///
@@ -86,12 +106,6 @@ pub use storage::DocumentText;
 ///     writing to the cache, and it takes responsibility for fetching
 ///     data from the network and updating the directory with new
 ///     directory information.
-///
-/// # Limitations
-///
-/// Because of portability issues in [`fslock::LockFile`], you might
-/// get weird results if you run two of these in the same process with
-/// the same underlying cache.
 pub struct DirMgr<R: Runtime> {
     /// Configuration information: where to find directories, how to
     /// validate them, and so on.
@@ -99,10 +113,6 @@ pub struct DirMgr<R: Runtime> {
     /// Handle to our sqlite cache.
     // XXXX I'd like to use an rwlock, but that's not feasible, since
     // rusqlite::Connection isn't Sync.
-    // TODO: Does this have to be a futures::Mutex?  I would rather have
-    // a rule that we never hold the guard for this mutex across an async
-    // suspend point.  But that will be hard to enforce until the
-    // `must_not_suspend` lint is in stable.
     store: Mutex<SqliteStore>,
     /// Our latest sufficiently bootstrapped directory, if we have one.
     ///
@@ -110,6 +120,18 @@ pub struct DirMgr<R: Runtime> {
     /// users, and replace it once a new directory is bootstrapped.
     netdir: SharedMutArc<NetDir>,
 
+    /// A flag that gets set whenever the _consensus_ part of `netdir` has
+    /// changed.
+    netdir_consensus_changed: AtomicBool,
+
+    /// A flag that gets set whenever the _descriptors_ part of `netdir` has
+    /// changed without adding a new consensus.
+    netdir_descriptors_changed: AtomicBool,
+
+    /// A publisher handle, used to inform others about changes in the
+    /// status of this directory handle.
+    publisher: event::Publisher,
+
     /// A circuit manager, if this DirMgr supports downloading.
     circmgr: Option<Arc<CircMgr<R>>>,
 
@@ -133,9 +155,7 @@ impl<R: Runtime> DirMgr<R> {
         // TODO: add some way to return a directory that isn't up-to-date
         let _success = dirmgr.load_directory().await?;
 
-        dirmgr
-            .opt_netdir()
-            .ok_or_else(|| Error::DirectoryNotPresent.into())
+        dirmgr.opt_netdir().ok_or(Error::DirectoryNotPresent)
     }
 
     /// Return a current netdir, either loading it or bootstrapping it
@@ -171,10 +191,7 @@ impl<R: Runtime> DirMgr<R> {
         let dirmgr = Arc::new(DirMgr::from_config(config, runtime.clone(), Some(circmgr))?);
 
         // Try to load from the cache.
-        let have_directory = dirmgr
-            .load_directory()
-            .await
-            .context("Error loading cached directory")?;
+        let have_directory = dirmgr.load_directory().await?;
 
         let (mut sender, receiver) = if have_directory {
             info!("Loaded a good directory from cache.");
@@ -188,20 +205,30 @@ impl<R: Runtime> DirMgr<R> {
         // Whether we loaded or not, we now start downloading.
         let dirmgr_weak = Arc::downgrade(&dirmgr);
         runtime.spawn(async move {
-            // TODO: don't warn when these are Error::ManagerDropped.
+            // NOTE: This is a daemon task.  It should eventually get
+            // treated as one.
+
+            // TODO: don't warn when these are Error::ManagerDropped: that
+            // means that the DirMgr has been shut down.
             if let Err(e) = Self::reload_until_owner(&dirmgr_weak, &mut sender).await {
-                warn!("Unrecoverd error while waiting for bootstrap: {}", e);
+                warn!("Unrecovered error while waiting for bootstrap: {}", e);
             } else if let Err(e) = Self::download_forever(dirmgr_weak, sender).await {
                 warn!("Unrecovered error while downloading: {}", e);
             }
         })?;
 
         if let Some(receiver) = receiver {
-            let _ = receiver.await;
+            match receiver.await {
+                Ok(()) => {
+                    info!("We have enough information to build circuits.");
+                }
+                Err(_) => {
+                    warn!("Bootstrapping task exited before finishing.");
+                    return Err(Error::CantAdvanceState);
+                }
+            }
         }
 
-        info!("We have enough information to build circuits.");
-
         Ok(dirmgr)
     }
 
@@ -217,23 +244,36 @@ impl<R: Runtime> DirMgr<R> {
         on_complete: &mut Option<oneshot::Sender<()>>,
     ) -> Result<()> {
         let mut logged = false;
-        let mut bootstrapped = false;
-        let runtime = upgrade_weak_ref(weak)?.runtime.clone();
+        let mut bootstrapped;
+        let runtime;
+        {
+            let dirmgr = upgrade_weak_ref(weak)?;
+            runtime = dirmgr.runtime.clone();
+            bootstrapped = dirmgr.netdir.get().is_some();
+        }
 
         loop {
             {
                 let dirmgr = upgrade_weak_ref(weak)?;
-                if dirmgr.try_upgrade_to_readwrite().await? {
+                trace!("Trying to take ownership of the directory cache lock");
+                if dirmgr.try_upgrade_to_readwrite()? {
                     // We now own the lock!  (Maybe we owned it before; the
                     // upgrade_to_readwrite() function is idempotent.)  We can
                     // do our own bootstrapping.
+                    if logged {
+                        info!("The previous owning process has given up the lock. We are now in charge of managing the directory.");
+                    }
                     return Ok(());
                 }
             }
 
             if !logged {
                 logged = true;
-                info!("Another process is bootstrapping. Waiting till it finishes or exits.");
+                if bootstrapped {
+                    info!("Another process is managing the directory. We'll use its cache.");
+                } else {
+                    info!("Another process is bootstrapping the directory. Waiting till it finishes or exits.");
+                }
             }
 
             // We don't own the lock.  Somebody else owns the cache.  They
@@ -249,11 +289,15 @@ impl<R: Runtime> DirMgr<R> {
             // our state functions.
             {
                 let dirmgr = upgrade_weak_ref(weak)?;
+                trace!("Trying to load from the directory cache");
                 if dirmgr.load_directory().await? {
                     // Successfully loaded a bootstrapped directory.
                     if let Some(send_done) = on_complete.take() {
                         let _ = send_done.send(());
                     }
+                    if !bootstrapped {
+                        info!("The directory is now bootstrapped.");
+                    }
                     bootstrapped = true;
                 }
             }
@@ -287,7 +331,7 @@ impl<R: Runtime> DirMgr<R> {
 
             'retry_attempt: for _ in retry_config.attempts() {
                 let (newstate, recoverable_err) =
-                    bootstrap::download(Weak::clone(&weak), state, on_complete.take()).await?;
+                    bootstrap::download(Weak::clone(&weak), state, &mut on_complete).await?;
                 state = newstate;
 
                 if let Some(err) = recoverable_err {
@@ -317,7 +361,12 @@ impl<R: Runtime> DirMgr<R> {
                     "We failed {} times to bootstrap a directory. We're going to give up.",
                     retry_config.n_attempts()
                 );
-                return Err(Error::CantAdvanceState.into());
+                return Err(Error::CantAdvanceState);
+            } else {
+                // Report success, if appropriate.
+                if let Some(send_done) = on_complete.take() {
+                    let _ = send_done.send(());
+                }
             }
 
             let reset_at = state.reset_time();
@@ -334,7 +383,7 @@ impl<R: Runtime> DirMgr<R> {
         self.circmgr
             .as_ref()
             .map(Arc::clone)
-            .ok_or_else(|| Error::NoDownloadSupport.into())
+            .ok_or(Error::NoDownloadSupport)
     }
 
     /// Try to make this a directory manager with read-write access to its
@@ -343,8 +392,26 @@ impl<R: Runtime> DirMgr<R> {
     /// Return true if we got the lock, or if we already had it.
     ///
     /// Return false if another process has the lock
-    async fn try_upgrade_to_readwrite(&self) -> Result<bool> {
-        self.store.lock().await.upgrade_to_readwrite()
+    fn try_upgrade_to_readwrite(&self) -> Result<bool> {
+        self.store
+            .lock()
+            .expect("Directory storage lock poisoned")
+            .upgrade_to_readwrite()
+    }
+
+    /// Return a reference to the store, if it is currently read-write.
+    fn store_if_rw(&self) -> Option<&Mutex<SqliteStore>> {
+        let rw = !self
+            .store
+            .lock()
+            .expect("Directory storage lock poisoned")
+            .is_readonly();
+        // A race-condition is possible here, but I believe it's harmless.
+        if rw {
+            Some(&self.store)
+        } else {
+            None
+        }
     }
 
     /// Construct a DirMgr from a DirMgrConfig.
@@ -356,10 +423,16 @@ impl<R: Runtime> DirMgr<R> {
         let readonly = circmgr.is_none();
         let store = Mutex::new(config.open_sqlite_store(readonly)?);
         let netdir = SharedMutArc::new();
+        let netdir_consensus_changed = AtomicBool::new(false);
+        let netdir_descriptors_changed = AtomicBool::new(false);
+        let publisher = event::Publisher::new();
         Ok(DirMgr {
             config,
             store,
             netdir,
+            netdir_consensus_changed,
+            netdir_descriptors_changed,
+            publisher,
             circmgr,
             runtime,
         })
@@ -370,8 +443,6 @@ impl<R: Runtime> DirMgr<R> {
     ///
     /// Return false if there is no such consensus.
     async fn load_directory(self: &Arc<Self>) -> Result<bool> {
-        //let store = &self.store;
-
         let state = state::GetConsensusState::new(Arc::downgrade(self), CacheUsage::CacheOnly)?;
         let _ = bootstrap::load(Arc::clone(self), Box::new(state)).await?;
 
@@ -393,14 +464,31 @@ impl<R: Runtime> DirMgr<R> {
         self.opt_netdir().expect("DirMgr was not bootstrapped!")
     }
 
-    /// Try to load the text of a signle document described by `doc` from
+    /// Return a new asynchronous stream about events taking place with
+    /// this directory manager.
+    ///
+    /// The caller must regularly process events from this stream to
+    /// prevent it from blocking.
+    pub fn events(&self) -> impl futures::Stream<Item = DirEvent> {
+        self.publisher.subscribe()
+    }
+
+    /// Try to load the text of a single document described by `doc` from
     /// storage.
-    pub async fn text(&self, doc: &DocId) -> Result<Option<DocumentText>> {
+    pub fn text(&self, doc: &DocId) -> Result<Option<DocumentText>> {
+        use itertools::Itertools;
         let mut result = HashMap::new();
         let query = (*doc).into();
-        self.load_documents_into(&query, &mut result).await?;
-        if let Some((docid, doctext)) = result.into_iter().next() {
-            assert_eq!(&docid, doc);
+        self.load_documents_into(&query, &mut result)?;
+        let item = result.into_iter().at_most_one().map_err(|_| {
+            Error::CacheCorruption("Found more than one entry in storage for given docid")
+        })?;
+        if let Some((docid, doctext)) = item {
+            if &docid != doc {
+                return Err(Error::CacheCorruption(
+                    "Item from storage had incorrect docid.",
+                ));
+            }
             Ok(Some(doctext))
         } else {
             Ok(None)
@@ -411,26 +499,43 @@ impl<R: Runtime> DirMgr<R> {
     ///
     /// If many of the documents have the same type, this can be more
     /// efficient than calling [`text`](Self::text).
-    pub async fn texts<T>(&self, docs: T) -> Result<HashMap<DocId, DocumentText>>
+    pub fn texts<T>(&self, docs: T) -> Result<HashMap<DocId, DocumentText>>
     where
         T: IntoIterator<Item = DocId>,
     {
         let partitioned = docid::partition_by_type(docs);
         let mut result = HashMap::new();
         for (_, query) in partitioned.into_iter() {
-            self.load_documents_into(&query, &mut result).await?
+            self.load_documents_into(&query, &mut result)?;
         }
         Ok(result)
     }
 
+    /// If the consensus has changed, notify any subscribers.
+    // TODO: I don't like all the different places in `bootstrap`
+    // where we have to call this function.  Can we simplify it or
+    // clean it up somehow?  Maybe we can build some kind of intelligence into
+    // shared_ref?
+    pub(crate) async fn notify(&self) {
+        if self.netdir_consensus_changed.swap(false, Ordering::SeqCst) {
+            self.publisher.send(DirEvent::NewConsensus).await;
+        }
+        if self
+            .netdir_descriptors_changed
+            .swap(false, Ordering::SeqCst)
+        {
+            self.publisher.send(DirEvent::NewDescriptors).await;
+        }
+    }
+
     /// Load all the documents for a single DocumentQuery from the store.
-    async fn load_documents_into(
+    fn load_documents_into(
         &self,
         query: &DocQuery,
         result: &mut HashMap<DocId, DocumentText>,
     ) -> Result<()> {
         use DocQuery::*;
-        let store = self.store.lock().await;
+        let store = self.store.lock().expect("Directory storage lock poisoned");
         match query {
             LatestConsensus {
                 flavor,
@@ -438,9 +543,11 @@ impl<R: Runtime> DirMgr<R> {
             } => {
                 if *cache_usage == CacheUsage::MustDownload {
                     // Do nothing: we don't want a cached consensus.
+                    trace!("MustDownload is set; not checking for cached consensus.");
                 } else if let Some(c) =
                     store.latest_consensus(*flavor, cache_usage.pending_requirement())?
                 {
+                    trace!("Found a reasonable consensus in the cache");
                     let id = DocId::LatestConsensus {
                         flavor: *flavor,
                         cache_usage: *cache_usage,
@@ -477,12 +584,12 @@ impl<R: Runtime> DirMgr<R> {
     ///
     /// This conversion has to be a function of the dirmgr, since it may
     /// require knowledge about our current state.
-    async fn query_into_requests(&self, q: DocQuery) -> Result<Vec<ClientRequest>> {
+    fn query_into_requests(&self, q: DocQuery) -> Result<Vec<ClientRequest>> {
         let mut res = Vec::new();
         for q in q.split_for_download() {
             match q {
                 DocQuery::LatestConsensus { flavor, .. } => {
-                    res.push(self.make_consensus_request(flavor).await?);
+                    res.push(self.make_consensus_request(flavor)?);
                 }
                 DocQuery::AuthCert(ids) => {
                     res.push(ClientRequest::AuthCert(ids.into_iter().collect()));
@@ -500,10 +607,11 @@ impl<R: Runtime> DirMgr<R> {
 
     /// Construct an appropriate ClientRequest to download a consensus
     /// of the given flavor.
-    async fn make_consensus_request(&self, flavor: ConsensusFlavor) -> Result<ClientRequest> {
+    fn make_consensus_request(&self, flavor: ConsensusFlavor) -> Result<ClientRequest> {
+        #![allow(clippy::unnecessary_wraps)]
         let mut request = tor_dirclient::request::ConsensusRequest::new(flavor);
 
-        let r = self.store.lock().await;
+        let r = self.store.lock().expect("Directory storage lock poisoned");
         match r.latest_consensus_meta(flavor) {
             Ok(Some(meta)) => {
                 request.set_last_consensus_date(meta.lifetime().valid_after());
@@ -525,12 +633,12 @@ impl<R: Runtime> DirMgr<R> {
     /// Currently, this handles expanding consensus diffs, and nothing
     /// else.  We do it at this stage of our downloading operation
     /// because it requires access to the store.
-    async fn expand_response_text(&self, req: &ClientRequest, text: String) -> Result<String> {
+    fn expand_response_text(&self, req: &ClientRequest, text: String) -> Result<String> {
         if let ClientRequest::Consensus(req) = req {
             if tor_consdiff::looks_like_diff(&text) {
                 if let Some(old_d) = req.old_consensus_digests().next() {
                     let db_val = {
-                        let s = self.store.lock().await;
+                        let s = self.store.lock().expect("Directory storage lock poisoned");
                         s.consensus_by_sha3_digest_of_signed_part(old_d)?
                     };
                     if let Some((old_consensus, meta)) = db_val {
@@ -544,7 +652,9 @@ impl<R: Runtime> DirMgr<R> {
                         return Ok(new_consensus.to_string());
                     }
                 }
-                return Err(Error::Unwanted("Received a consensus diff we did not ask for").into());
+                return Err(Error::Unwanted(
+                    "Received a consensus diff we did not ask for",
+                ));
             }
         }
         Ok(text)
@@ -574,7 +684,6 @@ enum Readiness {
 /// Resetting happens when this state needs to go back to an initial
 /// state in order to start over -- either because of an error or
 /// because the information it has downloaded is no longer timely.
-#[async_trait]
 trait DirState: Send {
     /// Return a human-readable description of this state.
     fn describe(&self) -> String;
@@ -594,13 +703,20 @@ trait DirState: Send {
     fn can_advance(&self) -> bool;
     /// Add one or more documents from our cache; returns 'true' if there
     /// was any change in this state.
-    fn add_from_cache(&mut self, docs: HashMap<DocId, DocumentText>) -> Result<bool>;
+    ///
+    /// If `storage` is provided, then we should write any state changes into
+    /// it.
+    fn add_from_cache(
+        &mut self,
+        docs: HashMap<DocId, DocumentText>,
+        storage: Option<&Mutex<SqliteStore>>,
+    ) -> Result<bool>;
 
     /// Add information that we have just downloaded to this state; returns
     /// 'true' if there as any change in this state.
     ///
     /// This method receives a copy of the original request, and
-    /// should reject any documents that do not purtain to it.
+    /// should reject any documents that do not pertain to it.
     ///
     /// If `storage` is provided, then we should write any accepted documents
     /// into `storage` so they can be saved in a cache.
@@ -609,7 +725,7 @@ trait DirState: Send {
     // TODO: It would be better to not have this function be async,
     // once the `must_not_suspend` lint is stable.
     // TODO: this should take a "DirSource" too.
-    async fn add_from_download(
+    fn add_from_download(
         &mut self,
         text: &str,
         request: &ClientRequest,
@@ -631,5 +747,5 @@ trait DirState: Send {
 /// Try to upgrade a weak reference to a DirMgr, and give an error on
 /// failure.
 fn upgrade_weak_ref<T>(weak: &Weak<T>) -> Result<Arc<T>> {
-    Weak::upgrade(weak).ok_or_else(|| Error::ManagerDropped.into())
+    Weak::upgrade(weak).ok_or(Error::ManagerDropped)
 }
