@@ -17,10 +17,9 @@ use std::convert::TryInto;
 use std::path::{self, Path, PathBuf};
 use std::time::SystemTime;
 
-use anyhow::Context;
-use chrono::prelude::*;
-use chrono::Duration as CDuration;
 use rusqlite::{params, OpenFlags, OptionalExtension, Transaction};
+use time::OffsetDateTime;
+use tracing::trace;
 
 #[cfg(target_family = "unix")]
 use std::os::unix::fs::DirBuilderExt;
@@ -40,8 +39,6 @@ pub(crate) struct SqliteStore {
     ///
     /// (sqlite supports that with connection locking, but we want to
     /// be a little more coarse-grained here)
-    // XXXX This can behave oddly or fail if this process already has
-    // XXXX another instance of this file; see fslock documentation.
     lockfile: Option<fslock::LockFile>,
 }
 
@@ -69,20 +66,13 @@ impl SqliteStore {
         let blobpath = path.join("dir_blobs/");
         let lockpath = path.join("dir.lock");
 
-        #[cfg(target_family = "unix")]
         if !readonly {
-            std::fs::DirBuilder::new()
-                .recursive(true)
-                .mode(0o700)
-                .create(&blobpath)
-                .with_context(|| format!("Creating directory at {:?}", &blobpath))?;
-        }
-        #[cfg(not(target_family = "unix"))]
-        if !readonly {
-            std::fs::DirBuilder::new()
-                .recursive(true)
-                .create(&blobpath)
-                .with_context(|| format!("Creating directory at {:?}", &blobpath))?;
+            let mut builder = std::fs::DirBuilder::new();
+            #[cfg(target_family = "unix")]
+            builder.mode(0o700);
+            builder.recursive(true).create(&blobpath).map_err(|err| {
+                Error::StorageError(format!("Creating directory at {:?}: {}", &blobpath, err))
+            })?;
         }
 
         let mut lockfile = fslock::LockFile::open(&lockpath)?;
@@ -132,14 +122,19 @@ impl SqliteStore {
 
     /// Try to upgrade from a read-only connection to a read-write connection.
     ///
-    /// Return true on succcess; false if another process had the lock.
+    /// Return true on success; false if another process had the lock.
     pub(crate) fn upgrade_to_readwrite(&mut self) -> Result<bool> {
         if self.is_readonly() && self.sql_path.is_some() {
-            let lf = self.lockfile.as_mut().unwrap();
+            let lf = self
+                .lockfile
+                .as_mut()
+                .expect("No lockfile open; cannot upgrade to read-write storage");
             if !lf.try_lock()? {
                 // Somebody else has the lock.
                 return Ok(false);
             }
+            // Unwrap should be safe due to parent `.is_some()` check
+            #[allow(clippy::unwrap_used)]
             match rusqlite::Connection::open(self.sql_path.as_ref().unwrap()) {
                 Ok(conn) => {
                     self.conn = conn;
@@ -186,7 +181,7 @@ impl SqliteStore {
             tx.commit()?;
             return Ok(());
         } else if readable_by > SCHEMA_VERSION {
-            return Err(Error::UnrecognizedSchema.into());
+            return Err(Error::UnrecognizedSchema);
         }
 
         // rolls back the transaction, but nothing was done.
@@ -234,7 +229,7 @@ impl SqliteStore {
             .components()
             .all(|c| matches!(c, path::Component::Normal(_)))
         {
-            return Err(Error::CacheCorruption("Invalid path in database").into());
+            return Err(Error::CacheCorruption("Invalid path in database"));
         }
 
         let mut result = self.path.clone();
@@ -242,15 +237,19 @@ impl SqliteStore {
         Ok(result)
     }
 
-    /// Read a blob from disk, mmapping it if possible.
+    /// Read a blob from disk, mapping it if possible.
     fn read_blob<P>(&self, path: P) -> Result<InputString>
     where
         P: AsRef<Path>,
     {
         let path = path.as_ref();
         let full_path = self.blob_fname(path)?;
-        InputString::load(&full_path)
-            .with_context(|| format!("Loading blob {:?} from storage at {:?}", path, full_path))
+        InputString::load(&full_path).map_err(|err| {
+            Error::StorageError(format!(
+                "Loading blob {:?} from storage at {:?}: {}",
+                path, full_path, err
+            ))
+        })
     }
 
     /// Write a file to disk as a blob, and record it in the ExtDocs table.
@@ -263,7 +262,7 @@ impl SqliteStore {
         doctype: &str,
         dtype: &str,
         digest: &[u8],
-        expires: DateTime<Utc>,
+        expires: OffsetDateTime,
     ) -> Result<SavedBlobHandle<'_>> {
         let digest = hex::encode(digest);
         let digeststr = format!("{}-{}", dtype, digest);
@@ -292,7 +291,7 @@ impl SqliteStore {
         doctype: &str,
         dtype: &str,
         digest: &[u8],
-        expires: DateTime<Utc>,
+        expires: OffsetDateTime,
     ) -> Result<String> {
         let h = self.save_blob_internal(contents, doctype, dtype, digest, expires)?;
         let SavedBlobHandle {
@@ -318,13 +317,16 @@ impl SqliteStore {
         let lifetime = cmeta.lifetime();
         let sha3_of_signed = cmeta.sha3_256_of_signed();
         let sha3_of_whole = cmeta.sha3_256_of_whole();
-        let valid_after: DateTime<Utc> = lifetime.valid_after().into();
-        let fresh_until: DateTime<Utc> = lifetime.fresh_until().into();
-        let valid_until: DateTime<Utc> = lifetime.valid_until().into();
+        let valid_after: OffsetDateTime = lifetime.valid_after().into();
+        let fresh_until: OffsetDateTime = lifetime.fresh_until().into();
+        let valid_until: OffsetDateTime = lifetime.valid_until().into();
+
+        /// How long to keep a consensus around after it has expired
+        const CONSENSUS_LIFETIME: time::Duration = time::Duration::days(4);
 
         // After a few days have passed, a consensus is no good for
         // anything at all, not even diffs.
-        let expires = valid_until + CDuration::days(4);
+        let expires = valid_until + CONSENSUS_LIFETIME;
 
         let doctype = format!("con:{}", flavor.name());
 
@@ -370,7 +372,7 @@ impl SqliteStore {
     /// Return the valid-after time for the latest non non-pending consensus,
     #[cfg(test)]
     // We should revise the tests to use latest_consensus_meta instead.
-    fn latest_consensus_time(&self, flavor: ConsensusFlavor) -> Result<Option<DateTime<Utc>>> {
+    fn latest_consensus_time(&self, flavor: ConsensusFlavor) -> Result<Option<OffsetDateTime>> {
         Ok(self
             .latest_consensus_meta(flavor)?
             .map(|m| m.lifetime().valid_after().into()))
@@ -387,7 +389,8 @@ impl SqliteStore {
         flavor: ConsensusFlavor,
         pending: Option<bool>,
     ) -> Result<Option<InputString>> {
-        let rv: Option<(DateTime<Utc>, DateTime<Utc>, String)>;
+        trace!(?flavor, ?pending, "Loading latest consensus from cache");
+        let rv: Option<(OffsetDateTime, OffsetDateTime, String)>;
         rv = match pending {
             None => self
                 .conn
@@ -418,7 +421,9 @@ impl SqliteStore {
         {
             Ok(text)
         } else {
-            Err(Error::CacheCorruption("couldn't find a consensus we thought we had.").into())
+            Err(Error::CacheCorruption(
+                "couldn't find a consensus we thought we had.",
+            ))
         }
     }
 
@@ -449,7 +454,8 @@ impl SqliteStore {
         let digest = format!("sha3-256-{}", d);
 
         let tx = self.conn.transaction()?;
-        tx.execute(MARK_CONSENSUS_NON_PENDING, params![digest])?;
+        let n = tx.execute(MARK_CONSENSUS_NON_PENDING, params![digest])?;
+        trace!("Marked {} consensuses usable", n);
         tx.commit()?;
 
         Ok(())
@@ -478,8 +484,8 @@ impl SqliteStore {
             let ids = meta.key_ids();
             let id_digest = hex::encode(ids.id_fingerprint.as_bytes());
             let sk_digest = hex::encode(ids.sk_fingerprint.as_bytes());
-            let published: DateTime<Utc> = meta.published().into();
-            let expires: DateTime<Utc> = meta.expires().into();
+            let published: OffsetDateTime = meta.published().into();
+            let expires: OffsetDateTime = meta.expires().into();
             stmt.execute(params![id_digest, sk_digest, published, expires, content])?;
         }
         stmt.finalize()?;
@@ -568,7 +574,7 @@ impl SqliteStore {
     {
         let tx = self.conn.transaction()?;
         let mut stmt = tx.prepare(UPDATE_MD_LISTED)?;
-        let when: DateTime<Utc> = when.into();
+        let when: OffsetDateTime = when.into();
 
         for md_digest in input.into_iter() {
             let h_digest = hex::encode(md_digest);
@@ -586,7 +592,7 @@ impl SqliteStore {
     where
         I: IntoIterator<Item = (&'a str, &'a MdDigest)>,
     {
-        let when: DateTime<Utc> = when.into();
+        let when: OffsetDateTime = when.into();
 
         let tx = self.conn.transaction()?;
         let mut stmt = tx.prepare(INSERT_MD)?;
@@ -610,7 +616,7 @@ impl SqliteStore {
         let mut stmt = tx.prepare(INSERT_RD)?;
 
         for (content, when, rd_digest) in input.into_iter() {
-            let when: DateTime<Utc> = when.into();
+            let when: OffsetDateTime = when.into();
             let h_digest = hex::encode(rd_digest);
             stmt.execute(params![h_digest, when, content])?;
         }
@@ -669,7 +675,7 @@ impl Drop for Unlinker {
 fn digest_from_hex(s: &str) -> Result<[u8; 32]> {
     hex::decode(s)?
         .try_into()
-        .map_err(|_| Error::CacheCorruption("Invalid digest in database").into())
+        .map_err(|_| Error::CacheCorruption("Invalid digest in database"))
 }
 
 /// Convert a hexadecimal sha3-256 "digest string" as used in the
@@ -678,18 +684,18 @@ fn digest_from_dstr(s: &str) -> Result<[u8; 32]> {
     if let Some(stripped) = s.strip_prefix("sha3-256-") {
         hex::decode(stripped)?
             .try_into()
-            .map_err(|_| Error::CacheCorruption("Invalid digest in database").into())
+            .map_err(|_| Error::CacheCorruption("Invalid digest in database"))
     } else {
-        Err(Error::CacheCorruption("Invalid digest in database").into())
+        Err(Error::CacheCorruption("Invalid digest in database"))
     }
 }
 
 /// Create a ConsensusMeta from a `Row` returned by one of
 /// `FIND_LATEST_CONSENSUS_META` or `FIND_CONSENSUS_AND_META_BY_DIGEST`.
 fn cmeta_from_row(row: &rusqlite::Row<'_>) -> Result<ConsensusMeta> {
-    let va: DateTime<Utc> = row.get(0)?;
-    let fu: DateTime<Utc> = row.get(1)?;
-    let vu: DateTime<Utc> = row.get(2)?;
+    let va: OffsetDateTime = row.get(0)?;
+    let fu: OffsetDateTime = row.get(1)?;
+    let vu: OffsetDateTime = row.get(2)?;
     let d_signed: String = row.get(3)?;
     let d_all: String = row.get(4)?;
     let lifetime = Lifetime::new(va.into(), fu.into(), vu.into())?;
@@ -858,7 +864,7 @@ const INSERT_EXTDOC: &str = "
   VALUES ( ?, datetime('now'), ?, ?, ? );
 ";
 
-/// Qury: Add a new consensus.
+/// Query: Add a new consensus.
 const INSERT_CONSENSUS: &str = "
   INSERT OR REPLACE INTO Consensuses
     ( valid_after, fresh_until, valid_until, flavor, pending, sha3_of_signed_part, digest )
@@ -919,12 +925,14 @@ const DROP_OLD_CONSENSUSES: &str = "
 
 #[cfg(test)]
 mod test {
+    #![allow(clippy::unwrap_used)]
     use super::*;
     use hex_literal::hex;
-    use tempdir::TempDir;
+    use tempfile::{tempdir, TempDir};
+    use time::ext::NumericalDuration;
 
     fn new_empty() -> Result<(TempDir, SqliteStore)> {
-        let tmp_dir = TempDir::new("arti-nd").unwrap();
+        let tmp_dir = tempdir().unwrap();
         let sql_path = tmp_dir.path().join("db.sql");
         let conn = rusqlite::Connection::open(&sql_path)?;
         let store = SqliteStore::from_conn(conn, &tmp_dir)?;
@@ -934,7 +942,7 @@ mod test {
 
     #[test]
     fn init() -> Result<()> {
-        let tmp_dir = TempDir::new("arti-nd").unwrap();
+        let tmp_dir = tempdir().unwrap();
         let sql_path = tmp_dir.path().join("db.sql");
         // Initial setup: everything should work.
         {
@@ -983,8 +991,8 @@ mod test {
     fn blobs() -> Result<()> {
         let (tmp_dir, mut store) = new_empty()?;
 
-        let now = Utc::now();
-        let one_week = CDuration::weeks(1);
+        let now = OffsetDateTime::now_utc();
+        let one_week = 1.weeks();
 
         let fname1 = store.save_blob(
             b"Hello world",
@@ -1044,8 +1052,8 @@ mod test {
         use tor_netdoc::doc::netstatus;
 
         let (_tmp_dir, mut store) = new_empty()?;
-        let now = Utc::now();
-        let one_hour = CDuration::hours(1);
+        let now = OffsetDateTime::now_utc();
+        let one_hour = 1.hours();
 
         assert_eq!(
             store.latest_consensus_time(ConsensusFlavor::Microdesc)?,
@@ -1142,8 +1150,8 @@ mod test {
     #[test]
     fn authcerts() -> Result<()> {
         let (_tmp_dir, mut store) = new_empty()?;
-        let now = Utc::now();
-        let one_hour = CDuration::hours(1);
+        let now = OffsetDateTime::now_utc();
+        let one_hour = 1.hours();
 
         let keyids = AuthCertKeyIds {
             id_fingerprint: [3; 20].into(),
@@ -1154,11 +1162,11 @@ mod test {
             sk_fingerprint: [3; 20].into(),
         };
 
-        let m1 = AuthCertMeta::new(keyids.clone(), now.into(), (now + one_hour * 24).into());
+        let m1 = AuthCertMeta::new(keyids, now.into(), (now + one_hour * 24).into());
 
         store.store_authcerts(&[(m1, "Pretend this is a cert")])?;
 
-        let certs = store.authcerts(&[keyids.clone(), keyids2])?;
+        let certs = store.authcerts(&[keyids, keyids2])?;
         assert_eq!(certs.len(), 1);
         assert_eq!(certs.get(&keyids).unwrap(), "Pretend this is a cert");
 
@@ -1169,21 +1177,22 @@ mod test {
     fn microdescs() -> Result<()> {
         let (_tmp_dir, mut store) = new_empty()?;
 
-        let now = Utc::now();
-        let one_day = CDuration::days(1);
+        let now = OffsetDateTime::now_utc();
+        let one_day = 1.days();
 
         let d1 = [5_u8; 32];
         let d2 = [7; 32];
         let d3 = [42; 32];
         let d4 = [99; 32];
 
+        let long_ago: OffsetDateTime = now - one_day * 100;
         store.store_microdescs(
             vec![
                 ("Fake micro 1", &d1),
                 ("Fake micro 2", &d2),
                 ("Fake micro 3", &d3),
             ],
-            (now - one_day * 100).into(),
+            long_ago.into(),
         )?;
 
         store.update_microdescs_listed(&[d2], now.into())?;
@@ -1208,9 +1217,9 @@ mod test {
     fn routerdescs() -> Result<()> {
         let (_tmp_dir, mut store) = new_empty()?;
 
-        let now = Utc::now();
-        let one_day = CDuration::days(1);
-        let long_ago = now - one_day * 100;
+        let now = OffsetDateTime::now_utc();
+        let one_day = 1.days();
+        let long_ago: OffsetDateTime = now - one_day * 100;
         let recently = now - one_day;
 
         let d1 = [5_u8; 20];
@@ -1242,7 +1251,7 @@ mod test {
 
     #[test]
     fn from_path_rw() -> Result<()> {
-        let tmp = TempDir::new("arti-from-path").unwrap();
+        let tmp = tempdir().unwrap();
 
         // Nothing there: can't open read-only
         let r = SqliteStore::from_path(tmp.path(), true);
